import os


DEFAULT_MAX_WORKERS = 1


PUBLIC_LEADERBOARD_MINIMUM_FEEDBACK_COUNT = 0


SUBMISSION_CONSTANT_FIELD_DEFAULTS = {
    'is_custom_reward': False,
    'elo_rating': float('nan'),
    'num_battles': 0,
    'num_wins': 0,
    'model_num_parameters': float('nan'),
}


LEADERBOARD_INDIVIDUAL_RANK_PARAMS = [
    dict(value_column='elo_rating', rank_column='elo_rank', ascending=False),
]


LEADERBOARD_OVERALL_RANK_PARAMS = [
    dict(
        from_rank_columns=['elo_rank',],
        overall_score_column='overall_user_score', overall_rank_column='overall_user_rank'
    ),
]


LEADERBOARD_FORMAT_CONFIGS = {}


LEADERBOARD_FORMAT_CONFIGS['meval'] = {
    "output_columns": [
        'developer_uid',
        'model_name',
        'is_custom_reward',
        "overall_meval_score",
        'elo_rating',
        'num_battles',
        'num_wins',
        'size',
        'status',
        'submission_id',
    ],
    "sort_params": {
        "by": "overall_meval_score",
        "ascending": True
    },
}


LEADERBOARD_FORMAT_CONFIGS['user'] = {
    "output_columns": [
        'developer_uid',
        'model_name',
        'elo_rating',
        'elo_rank',
        'overall_user_score',
        'num_battles',
        'num_wins',
        'is_custom_reward',
        'submission_id',
        'size',
    ],
    "sort_params": {
        "by": "overall_user_score",
        "ascending": True
    }
}



from chaiverse import formatters


def get_formatter_map():
    cls_names = dir(formatters)
    formatter_classes = {
        cls_name: getattr(formatters, cls_name)
        for cls_name in cls_names
        if _is_formatter(cls_name)}
    return formatter_classes


def print_formatter_info(formatter_name):
    formatter = get_formatter(formatter_name)
    for field_name, model_field in formatter.__fields__.items():
        print(field_name, ':', repr(model_field.default))


def get_formatter(formatter_name):
    formatter_classes = get_formatter_map()
    formatter = formatter_classes.get(formatter_name)
    if not formatter:
        raise NameError(f'{formatter_name} is not a valid formatter for Chaiverse. Check chaiverse.formatters for the supported ones.')
    return formatter


def _is_formatter(cls_name):
    cls = getattr(formatters, cls_name)
    is_not_base_class = (cls_name != 'PromptFormatter')
    is_subclass = isinstance(cls, type) and issubclass(cls, formatters._PromptFormatter)
    return is_not_base_class and is_subclass




from contextlib import contextmanager
import re


REGEXES = [
    # AWS Secret Key Detector
    re.compile(r'[A-Za-z0-9]{43}'),
    # HuggingFace Token Detector
    re.compile(r'hf_[A-Za-z0-9_]{34}'),
    # Chaiverse Token Detector
    re.compile(r'CR_[A-Za-z0-9_]{32}'),
    # Github Token Detector
    re.compile(r'(?:ghp|gho|ghu|ghs|ghr)_[A-Za-z0-9_]{36}'),
    # Gitlab Token Detector
    re.compile(r'(?:gldt|glpat|glrt|glptt|glft|gmimt|glagent)-[A-Za-z0-9]{3}-[A-Za-z]{10}_[a-z]{5}'),
    # Discord bot Token Detector
    re.compile(r'[MNO][a-zA-Z\d_-]{23,25}\.[a-zA-Z\d_-]{6}\.[a-zA-Z\d_-]{27}'),
]


def scrub_secrets(string: str, redaction=""):
    try:
        decoded_string = string.decode("utf-8") if isinstance(string, bytes) else string
        secrets = detect_secrets(decoded_string)
        for secret in secrets:
            decoded_string = decoded_string.replace(secret, redaction)
        string = str.encode(decoded_string) if isinstance(string, bytes) else decoded_string
    except UnicodeDecodeError:
        pass
    return string


def detect_secrets(string: str):
    regex_matches = [regex.search(str(string)) for regex in REGEXES]
    secrets = [match.group() for match in regex_matches if match]
    return secrets
